-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local cityZones =
{
  [L["Dalaran"]]         = true,
  [L["Darnassus"]]       = true,
  [L["Ironforge"]]       = true,
  [L["Orgrimmar"]]       = true,
  [L["Shattrath City"]]  = true,
  [L["Silvermoon City"]] = true,
  [L["Stormwind"]]       = true,
  [L["The Exodar"]]      = true,
  [L["Thunder Bluff"]]   = true,
  [L["Undercity"]]       = true
};

-- Used to build a mask for summoning critters
local critterMasks =
{
  ["Kalimdor"]         = 0x0001,
  ["Eastern Kingdoms"] = 0x0002,
  ["Outland"]          = 0x0004,
  ["Northrend"]        = 0x0008,
  ["city"]             = 0x0010,
  ["party"]            = 0x0020,
  ["raid"]             = 0x0040,
  ["arena"]            = 0x0080,
  ["pvp"]              = 0x0100,
};

-- Critters used as part of a quest
local questCritters =
{
  -- [31880] = -- Blood Elf Orphan Whistle
  {
    "all"
  },
  [31881] = -- Draenei Orphan Whistle
  {
    "all"
  },
  [30803] = -- Felhound Whistle
  {
    L["Hellfire Peninsula"]
  },
  [36865] = -- Golem Control Unit
  {
    L["Dun Argol"]
  },
  [36865] = -- Golem Control Unit
  {
    "all"
  },
  [18598] = -- Human Orphan Whistle
  {
    "all"
  },
  [46397] = -- Oracle Orphan Whistle
  {
    "all"
  },
  [18597] = -- Orcish Orphan Whistle
  {
    "all"
  },
  [34253] = -- Sizzling Ember
  {
    L["Throne of Kil'jaeden"]
  },
  [46362] =
  {
    L["Silithus"],
    L["Tanaris"],
    L["Un'Goro Crater"]
  },
  [68646] = -- Winterspring Cub
  {
    L["Winterspring"]
  },
  [46396] = -- Wolvar Orphan Whistle
  {
    "all"
  },
  [31815] = -- Zeppit's Crystal
  {
    L["Netherstorm"]
  }
};

local mountOptimalType =
{
  [64731] = 0x08 -- Sea Turtle
};

local mountSpells =
{
  [ 1066] = 0x08, -- Aquatic Form
  [33943] = 0x02, -- Flight Form
  [ 2645] = 0x01, -- Ghost Wolf
  [87840] = 0x01, -- Running Wild
  [40120] = 0x02, -- Swift Flight Form
  [  783] = 0x01, -- Travel Form
};

local mountSpellGroups =
{
  { 1066       },
  {40120, 33943},
  {  783       },
  { 2645       },
  {87840       }
};

local mountRestricted =
{
  [75207] = -- Abyssal Seahorse
  {
    [L["Shimmering Expanse"]] = 1,
    [L["Abyssal Depths"]]     = 1,
    [L["Kelp'thar Forest"]]   = 1
  },
  [25953] = -- Blue Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  },
  [26056] = -- Green Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  },
  [26054] = -- Red Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  },
  [26055] = -- Yellow Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  }
};

local zoneRestricted = {};

local mountHolidays =
{
  ["HallowsEnd"] =
  {
    [37011] = 1 -- Magic Broom
  },
  ["HallowsEndEx"] =
  {
    [33176] = 4 -- Flying Broom
  }
};

local mountIdealFlags =
{
  [75614] = 0x03, -- Celestial Steed
  [ 2645] = 0x00, -- Ghost Wolf
  [48025] = 0x0B, -- Headless Horseman's Mount
  [72286] = 0x03, -- Invincible
  [64731] = 0x08, -- Sea Turtle
  [  783] = 0x00, -- Travel Form
};

local ridingInfo =
{
  33388, -- Apprentice Riding
  33391, -- Journeyman Riding
  34090, -- Artisan Riding
  34091, -- Expert Riding
  90265  -- Master Riding
};

local restrictedZones = {};
local continentalFlight = {};
local isTravelForm = {};
local mountMacroText = "";
local ridingSkill = 0;
local hasClassRaceMounts = false;

StaticPopupDialogs["NIGGLES_COMPANIONS_CLEARALL"] =
{
  text         = "%s",
  button1      = OKAY,
  button2      = CANCEL,
  whileDead    = 1,
  OnAccept     = function(self, data)
    NigglesOptionsCompanionsClearAll(data);
  end,
  showAlert    = 1,
  timeout      = 0,
  exclusive    = 1,
  hideOnEscape = 1
};
StaticPopupDialogs["NIGGLES_COMPANIONS_SELECTALL"] =
{
  text         = "%s",
  button1      = OKAY,
  button2      = CANCEL,
  whileDead    = 1,
  OnAccept     = function(self, data)
    NigglesOptionsCompanionsSelectAll(data);
  end,
  showAlert    = 1,
  timeout      = 0,
  exclusive    = 1,
  hideOnEscape = 1
};

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function called when the contents of the player's bags changes. It searches
-- for any quest items used to summon critters
--
function NigglesCrittersBagUpdate()
  -- Clear the list of restricted zones
  wipe(restrictedZones);

  -- Search for any items used to summon quest critters
  for itemId, itemZones in pairs(questCritters) do
    if (GetItemCount(itemId, false, false) > 0) then
      if (itemZones[1] == "all") then
        restrictedZones["all"] = true;
        break;
      else
        for _, zoneName in ipairs(itemZones) do
          restrictedZones[zoneName] = true;
        end
      end
    end
  end

  return;
end

--
-- Function to set the mask used to determine if critters should be summoned
--
function NigglesCrittersSetMask()
  -- Reset the mask
  Niggles.critterMask = 0;

  -- Add flags for the normal zones
  if (NigglesDBC.settings.CrittersInKalimdor == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Kalimdor"]);
  end
  if (NigglesDBC.settings.CrittersInEKingdoms == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Eastern Kingdoms"]);
  end
  if (NigglesDBC.settings.CrittersInNorthrend == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Northrend"]);
  end
  if (NigglesDBC.settings.CrittersInOutland == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Outland"]);
  end

  -- Add flags for special types of zone
  if (NigglesDBC.settings.CrittersInCities == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["city"]);
  end
  if (NigglesDBC.settings.CrittersInDungeons == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["party"]);
  end
  if (NigglesDBC.settings.CrittersInRaids == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["raid"]);
  end
  if (NigglesDBC.settings.CrittersInArena == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["arena"]);
  end
  if (NigglesDBC.settings.CrittersInBG == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["pvp"]);
  end

  return;
end

--
-- Function to summon a random critter, if appropriate
--
function NigglesCritterSummon()
  -- Local Variables
  local critters = {};
  local summonCritter = true;
  local zoneContinent = "";
  local zoneMask = 0;
  local zoneName;
  local zoneType;

  -- Check if the previously summoned critter is still active
  if (Niggles.critterIdx > 0) then
    local _, _, _, _, isActive = GetCompanionInfo("CRITTER",
      Niggles.critterIdx);
    if (isActive == 1) then
      summonCritter = false;
    end
  end

  -- Get info for the zone the player is currently in
  if (summonCritter == true) then
    zoneName, zoneType = GetInstanceInfo();
    if (zoneType == "none") then
      zoneContinent = zoneName;
      zoneName      = GetRealZoneText();
      if (NigglesIsCity(zoneName) == true) then

        zoneMask = critterMasks["city"];
      elseif (critterMasks[zoneContinent] ~= nil) then
        zoneMask = critterMasks[zoneContinent];
      else
        summonCritter = false;
      end
    else
      if (critterMasks[zoneType] ~= nil) then
        zoneMask = critterMasks[zoneType];
      else
        summonCritter = false;
      end
    end
  end

  -- Check if any special critters might be summoned in the current area
  if ((summonCritter == true) and
      (next(restrictedZones) ~= nil)) then
    if ((restrictedZones["all"] == true) or
        (restrictedZones[zoneName] == true) or
        (restrictedZones[GetSubZoneText()] == true)) then
      summonCritter = false;
    end
  end

  -- Check if a critter needs to be summoned and can be
  if ((summonCritter == true) and
      (bit.band(Niggles.critterMask, zoneMask) == zoneMask) and
      (next(NigglesDBC.critters) ~= nil) and
      (IsMounted() == nil) and
      (IsFlying() == nil) and
      (IsFalling() == nil) and
      (IsStealthed() == nil) and
      (GetNumLootItems() == 0) and
      (UnitInVehicle("player") == nil) and
      (UnitUsingVehicle("player") == nil) and
      (UnitIsDeadOrGhost("player") == nil) and
      (UnitIsCharmed("player") == nil) and
      (UnitIsFeignDeath("player") == nil) and
      (UnitIsPossessed("player") == nil) and
      (UnitCastingInfo("player") == nil) and
      (UnitChannelInfo("player") == nil) and
      (UnitAura("player", L["Food"]) == nil) and
      (UnitAura("player", L["Drink"]) == nil) and
      (UnitAura("player", L["Feign Death"]) == nil) and
      (UnitAura("player", L["Camouflage"]) == nil) and
      (UnitAura("player", L["Herbouflage"]) == nil) and
      (UnitAura("player", L["Spirit of Redemption"]) == nil) and
      (GetSubZoneText() ~= L["Dun Argol"]) and
      (GetSubZoneText() ~= L["Throne of Kil'jaeden"]) and
      ((BarberShopFrame == nil) or (BarberShopFrame:IsVisible() == nil))) then
    -- Get the number of critters known by the player
    numCritters = GetNumCompanions("CRITTER");

    -- Work out which critters have been selected by the player
    for index = 1, numCritters do
      local _, _, spellId, _, isActive = GetCompanionInfo("CRITTER", index);
      if (NigglesDBC.critters[spellId] == 1) then
        critters[1+#critters] = index;
      end
      if (isActive == 1) then
        summonCritter = false;
        break;
      end
    end

    -- Call a random critter, if one isn't already summoned
    if ((summonCritter == true) and
        (next(critters) ~= nil)) then
      Niggles.critterIdx = math.random(#critters);
      CallCompanion("CRITTER", critters[Niggles.critterIdx]);
    end
  end

  return;
end

--
-- Function to check if a specified zone is a city
--
function NigglesIsCity(zoneName)
  return (cityZones[zoneName] ~= nil);
end

--
-- Function clear the macro of the hidden mount button after it has been
-- 'clicked'.
--
function NigglesMountPostClick(button)
  -- Clear the button's macro
  if (InCombatLockdown() == nil) then
    button:SetAttribute("macrotext", mountMacroText);
  end
end

--
-- Function to work out what to do when a macro calls '/click NigglesMount'.
-- This function is called before the button is 'clicked'. It works out what
-- to do based on the player's current status and changes the macro of the
-- hidden button accordingly.
--
function NigglesMountPreClick(button)
  -- Local Variables
  local activeHolidays;
  local currentContinent;
  local currentZone;
  local formIdx = GetShapeshiftForm();
  local isWintergraspBattle = false;
  local mapId;
  local mountFlags = nil;
  local randomList = {};
  local spellName = nil;
  local wintergraspName;
  local flagIdx = 1;

  -- Get the current continent
  mapId = GetCurrentMapAreaID();
  SetMapToCurrentZone();
  currentContinent = GetCurrentMapContinent();
  SetMapByID(mapId);

  -- Get the name of the current zone
  currentZone = GetRealZoneText();

  -- Get info about Wintergrasp
  _, wintergraspName, isWintergraspBattle = GetWorldPVPAreaInfo(1);

  -- Work out what to do, based on the player's current location and state
  if (UnitInVehicle("player")) then
    -- Exit any vehicle the player is in
    VehicleExit();
  elseif (IsMounted() == 1) then
    -- Dismount from any mount the player is on
    Dismount();
  elseif ((UnitAura("PLAYER", L["Herbouflage"]) == L["Herbouflage"]) and
          (InCombatLockdown() == nil)) then
    -- Cancel the aura
    button:SetAttribute("macrotext", "/cancelaura "..L["Herbouflage"]);
  elseif ((formIdx > 0) and
          (isTravelForm[formIdx] == true) and
          (InCombatLockdown() == nil)) then
    -- Cancel the current shapeshift form
    button:SetAttribute("macrotext", "/cancelform [form:"..formIdx.."]");
  elseif (IsSwimming()) then
    if (not IsModifierKeyDown()) then
      mountFlags = {0x08, 0x02, 0x01};
    else
      mountFlags = {0x02, 0x01};
    end
  elseif (IsFlyableArea()) then
    -- Work out if the player can fly
    if ((not IsModifierKeyDown()) and
        (continentalFlight[currentContinent] == true) and
        ((currentContinent ~= 4) or
         (currentZone ~= wintergraspName) or
         (isWintergraspBattle ~= true))) then
      mountFlags = {0x02, 0x01};
    else
      mountFlags = {0x01};
    end
  else
    mountFlags = {0x01};
  end

  -- Check if the player is in combat
  if (mountFlags ~= nil) then
    if (InCombatLockdown() == 1) then
      UIErrorsFrame:AddMessage(SPELL_FAILED_AFFECTING_COMBAT,
        1.0, 0.1, 0.1, 1.0);
    elseif (next(NigglesDBC.mounts) == nil) then
      UIErrorsFrame:AddMessage(L["No mounts selected"], 1.0, 0.1, 0.1, 1.0);
    else
      -- Check if holiday mounts should be used
      if (NigglesDBC.settings.MountsHoliday == true) then
        -- Build a list of holiday mounts
        activeHolidays = NigglesHolidaysGetActive();
        for _, holiday in ipairs(activeHolidays) do
          if (mountHolidays[holiday] ~= nil) then
            for itemId, itemSkill in pairs(mountHolidays[holiday]) do
              if ((IsUsableItem(itemId)) and 
                  (itemSkill <= ridingSkill)) then
                table.insert(randomList, itemId);
              end
            end
          end
        end

        -- Check if any suitable mounts were found
        if (next(randomList) ~= nil) then
          -- Choose a random mount
          spellName = GetItemInfo(randomList[math.random(#(randomList))]);
        end
      end

      -- Search for a non-holiday mount
      if (spellName == nil) then
        -- Search each specified mount type for a mount
        for _, currentType in ipairs(mountFlags) do
          -- Check if there are any mounts restricted to the current zone
          if (zoneRestricted[currentZone] ~= nil) then
            -- Search for selected restricted mounts with the current flag
            for _, spellId in ipairs(zoneRestricted[currentZone]) do
              if ((NigglesDBC.mounts[spellId] ~= nil) and
                  (bit.band(NigglesDBC.mounts[spellId], currentType) ~= 0) and
                  (IsUsableSpell(spellId))) then
                table.insert(randomList, spellId);
              end
            end
           
            -- Check if any suitable mounts were found
            if (next(randomList) ~= nil) then
              -- Choose a random mount
              spellName = GetSpellInfo(randomList[math.random(#(randomList))]);
              break;
            end
          end

          -- Search for unrestricted mounts that have the current flag
          if (spellName == nil) then
            -- Search for selected mounts with the current flag
            for spellId, spellFlags in pairs(NigglesDBC.mounts) do
              if ((bit.band(spellFlags, currentType) ~= 0) and
                  (IsUsableSpell(spellId)) and
                  (mountRestricted[spellId] == nil)) then
                table.insert(randomList, spellId);
              end
            end
          
            -- Check if any mounts were found
            if (next(randomList) ~= nil) then
              -- Choose a random mount
              spellName = GetSpellInfo(randomList[math.random(#(randomList))]);
              break;
            end
          end
        end
      end

      -- Update the button with the mount's spell name
      if (spellName ~= nil) then
        button:SetAttribute("macrotext",
          mountMacroText.."/cast "..spellName);
      else
        button:SetAttribute("macrotext", "");
        UIErrorsFrame:AddMessage(L["No suitable mount"], 1.0, 0.1, 0.1, 1.0);
      end
    end
  end

  return;
end

--
-- Function to initialise info for the mount system
--
function NigglesMountsInitialise()
  -- Work out if the player can have class/race mounts
  if ((Niggles.playerClass == "DRUID") or
      (Niggles.playerClass == "SHAMAN") or
      (Niggles.playerRace  == "Worgen")) then
    hasClassRaceMounts = true;
  else
    hasClassRaceMounts = false;
    NigglesDBC.settings.MountsClassRace = false;
  end

  -- Make a table Initialise zones with restricted mounts
  for spellId, zones in pairs(mountRestricted) do
    for zone, _ in pairs(zones) do
      if (zoneRestricted[zone] == nil) then
        zoneRestricted[zone] = {};
      end
      table.insert(zoneRestricted[zone], spellId);
    end
  end

  return;
end

--
-- Function called when the player's spells change
--
function NigglesMountsSpellsUpdate()
  -- Local Variables
  local formName;
  local numContinents = select("#", GetMapContinents());
  local numForms = GetNumShapeshiftForms();
  local spellName;

  -- Work out the player's riding skill
  ridingSkill = 0;
  for idx, spellId in ipairs(ridingInfo) do
    if (IsSpellKnown(spellId)) then
      ridingSkill = idx;
      break;
    end
  end

  -- Clear the continental flight table
  for idx = 1, numContinents do
    continentalFlight[idx] = false;
  end

  -- Check if the player has the skill to fly
  if (ridingSkill >= 3) then
    -- Check if the player can fly in Azeroth
    if (IsSpellKnown(90267)) then -- Flight Pilot's License
      continentalFlight[1] = true;
      continentalFlight[2] = true;
      continentalFlight[5] = true;
    end

    -- Set the flag that indicates the player can fly in Outland
    continentalFlight[3] = true;

    -- Check if the player can fly in Northrend
    if (IsSpellKnown(54197)) then -- Cold Weather Flying
      continentalFlight[4] = true;
    end
  end

  -- Work out which forms are travel forms
   wipe(isTravelForm);
   for spellId, _ in pairs(mountSpells) do
     spellName = GetSpellInfo(spellId);
     if (spellName ~= nil) then
       for formIdx = 1, numForms do
         _, formName = GetShapeshiftFormInfo(formIdx);
         if ((formName ~= nil) and
             (formName == spellName)) then
           isTravelForm[formIdx] = true;
           break;
         end
       end
     end
   end

  -- Build the macro text for the mount button
  mountMacroText = "";
  for idx = 1, numForms do
    if (isTravelForm[idx] == true) then
      mountMacroText = mountMacroText.."[combat,form:"..idx.."]";
    else
      isTravelForm[idx] = false;
    end
  end
  if (mountMacroText ~= "") then
    mountMacroText = "/cancelform "..mountMacroText.."\n";
  end

  return
end

--
-- Function to clear all selected companions/mounts
--
function NigglesOptionsCompanionsClearAll(companionType)
  -- Check the specified type is valid
  if ((companionType == "critters") or (companionType == "mounts")) then
    -- Clear all the selected companions
    wipe(Niggles.updatedOptions[companionType]);

    -- Update any frames that may be affected
    NigglesOptionsRefreshCompanions();
  end

  return;
end

--
-- Function to enable/disable frames in the Niggles Options Companions panel
--
function NigglesOptionsCompanionsEnableFrames()
  -- Local Variables
  local button;

  -- Enable or disable frames based on the updated options
  NigglesEnableFrames(Niggles.updatedOptions.settings.MountsEnabled,
    NigglesOptionsCompanionsMountsText,
    NigglesOptionsCompanionsMountsHoliday,
    NigglesOptionsCompanionsMountsClear,
    NigglesOptionsCompanionsMountsSelect);
  NigglesEnableFrames(
    ((Niggles.updatedOptions.settings.MountsEnabled) and (hasClassRaceMounts)),
    NigglesOptionsCompanionsMountsClassRace);
  NigglesEnableFrames(Niggles.updatedOptions.settings.CrittersEnabled,
    NigglesOptionsCompanionsCrittersText,
    NigglesOptionsCompanionsCrittersInKalimdor,
    NigglesOptionsCompanionsCrittersInEKingdoms,
    NigglesOptionsCompanionsCrittersInNorthrend,
    NigglesOptionsCompanionsCrittersInOutland,
    NigglesOptionsCompanionsCrittersInCities,
    NigglesOptionsCompanionsCrittersInDungeons,
    NigglesOptionsCompanionsCrittersInRaids,
    NigglesOptionsCompanionsCrittersInArenas,
    NigglesOptionsCompanionsCrittersInBG,
    NigglesOptionsCompanionsCrittersClear,
    NigglesOptionsCompanionsCrittersSelect);

  -- Enable or disable the class/race buttons based on the updated options
  for idx = 1, #mountSpellGroups do
    button = _G["NigglesOptionsCompanionsClassRaceButton"..idx];
    if (button ~= nil) then
      if ((Niggles.updatedOptions.settings.MountsEnabled   == true) and
          (Niggles.updatedOptions.settings.MountsClassRace == true)) then
        button:Enable();
      else
        button:Disable();
      end
    end
  end

  return;
end

--
-- Function to refresh the settings for the Interface Options
-- 'Mounts & Companions' panel
--
function NigglesOptionsRefreshCompanions()
  -- Local Variables
  local parentName = "NigglesOptionsCompanions";
  local buttonName = parentName.."ClassRaceButton";
  local knownSpellId;
  local spellIcon;
  
  -- Initialise all the class/race buttons to the correct state
  for idx = 1, #mountSpellGroups do
    -- Create the button, if required
    button = _G[buttonName..idx];
    if (button == nil) then
      button = CreateFrame("CHECKBUTTON", buttonName..idx, _G[parentName],
        "NigglesClassRaceButtonTemplate");
      button:SetPoint("TOPLEFT", _G[buttonName..(idx-1)], "TOPRIGHT", 3, 0);
    end

    -- Check the button exists
    if (button ~= nil) then
      -- Search for a known spell within the group
      button.spellId = nil;
      for _, spellId in ipairs(mountSpellGroups[idx]) do
        if (IsSpellKnown(spellId)) then
          button.spellId = spellId;
          break;
        end
      end

      -- If no spell in the group is known...
      if (button.spellId == nil) then
        -- ...use the first one in the group
        button.spellId = mountSpellGroups[idx][1];
      end

      -- Update the button's icon and state
      _, _, spellIcon = GetSpellInfo(button.spellId);
      button:SetNormalTexture(spellIcon);
      if (IsSpellKnown(button.spellId)) then
        button:Enable();
        button:SetChecked(Niggles.updatedOptions.mounts[button.spellId]);
      else
        button:Disable();
        button:SetChecked(false);
      end
      button.spellFlags = mountSpells[button.spellId];
    end
  end

  -- Initialise all the widgets to the correct state for the settings
  NigglesOptionsRefreshSettings(NigglesOptionsCompanions);

  -- Initialise all the frames to the correct state for the settings
  NigglesOptionsCompanionsEnableFrames();

  return;
end

--
-- Function to select all companions/mounts as selected
--
function NigglesOptionsCompanionsSelectAll(companionType)
  -- Local Variables
  local spellFlags;
  local spellId;
  local numCompanions;
  local idealFlags;

  -- Check the specified type is valid
  if (companionType == "critters") then
    -- Clear all selected critters
    wipe(Niggles.updatedOptions.critters);

    -- Select all know critters
    numCompanions = GetNumCompanions("CRITTER");
    for idx = 1, numCompanions do
      _, _, spellId = GetCompanionInfo("CRITTER", idx);
      if (spellId ~= nil) then
        Niggles.updatedOptions.critters[spellId] = 1;
      end
    end
  elseif (companionType == "mounts") then
    -- Clear all selected mounts
    wipe(Niggles.updatedOptions.mounts);

    -- Select all know mounts
    numCompanions = GetNumCompanions("MOUNT");
    for idx = 1, numCompanions do
      _, _, spellId, _, _, spellFlags = GetCompanionInfo("MOUNT", idx);
      if (spellId ~= nil) then
        -- Check if the mount has ideal flags specified for it
        if (mountIdealFlags[spellId] ~= nil) then
          -- Use the ideal flags
          if (mountIdealFlags[spellId] ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = mountIdealFlags[spellId];
          end
        else
          -- Set the mount's flags to select its optimal environment
          if (bit.band(spellFlags, 0x02) ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = 0x02;
          elseif (bit.band(spellFlags, 0x01) ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = 0x01;
          elseif (bit.band(spellFlags, 0x08) ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = 0x08;
          end
        end
      end
    end
   
    -- Select all known mount spells
    for spellId, spellFlags in pairs(mountSpells) do
      if (IsSpellKnown(spellId)) then
        -- Check if the spell has ideal flags specified for it
        if (mountIdealFlags[spellId] ~= nil) then
          -- Use the spell's ideal flags
          if (mountIdealFlags[spellId] ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = mountIdealFlags[spellId];
          end
        else
          -- Use the spell's standard flags
          Niggles.updatedOptions.mounts[spellId] = spellFlags;
        end
      end
    end
  end

  -- Update any frames that may be affected
  NigglesOptionsRefreshCompanions();

  return;
end

